import math
import random
from math import radians, degrees, atan2
from random import randrange, randint

import pyglet
import pymunk as pm
from pyglet.window import key
from pyglet.gl import *
from pymunk.vec2d import Vec2d

import sprite
import helper
from res import Images



class Cat(sprite.PmSprite):
    
    def __init__(self, img, window, space, lawn_mower, *args, **kwargs):
        sprite.PmSprite.__init__(self, img, window, space, *args, **kwargs)
        
        self.state = "explore"
        self.image = Images.cat_explore
        
        self.target_vec2d = Vec2d(0,0)
        self.lawn_mower = lawn_mower
        self.chain = None
                
        self.radius = self.width/2
        self.mass = 0.2
        self.inertia = pm.moment_for_circle(self.mass, self.radius, 0, Vec2d(0,0))
        self.move_speed = 1
        self.steer_speed = 2
        
        self.body = pm.Body(self.mass, self.inertia)
        self.position = self.body.position
        self.pm_shape = pm.Circle(self.body, self.radius, Vec2d(0,0))
        self.pm_shape.friction = 0.1

    
        
        
    def update(self,dt):
        if self.state == "explore":
            self.explore(dt)
            
        if self.state == "scared":
            self.scared(dt)
            
        if self.state == "caught":
            pass
        
        #self.body.angle = radians(self.get_angle_to(self.window.mpos))
        #self.wrap()
        self.position = self.body.position
        self.rotation = -degrees(self.body.angle)
        
        
        #print degrees(self.body.angle)
        
    def explore(self,dt):
        if self.body.position.get_distance(self.target_vec2d) < 15:
            self.body.position = self.target_vec2d
            
        if not self.collide(self.target_vec2d):
            if self.body.angle != radians(self.get_angle_to(self.target_vec2d)):
                self.body.angle = radians(self.get_angle_to(self.target_vec2d))
            f = self.body._bodycontents.rot * self.move_speed
            self.body.apply_impulse(Vec2d(f.x, f.y) , (0,0))
            
        if self.body.position.get_distance(self.lawn_mower.body.position) < 100:
            self.set_new_target()
            self.state = "scared"
            
                
        if self.collide(self.target_vec2d):
            self.set_new_target()
            
    def scared(self,dt):
        if self.body.position.get_distance(self.target_vec2d) < 15:
            self.body.position = self.target_vec2d
        
        if not self.collide(self.target_vec2d):
            if self.body.angle != radians(self.get_angle_to(self.target_vec2d)):
                self.body.angle = radians(self.get_angle_to(self.target_vec2d))
            f = self.body._bodycontents.rot * self.move_speed
            self.body.apply_impulse(Vec2d(f.x, f.y) , (0,0))
            
        if self.collide(self.target_vec2d):
            self.state = "explore"    
        
        
            
    def chain_to(self, other):
        self.state = "caught"
        self.chain = pm.PivotJoint(self.body, other.body, other.body.position)
        self.image = Images.cat_caught
        self.space.add(self.chain)
            
    
    def get_angle_to(self,other):
        xs = other.x - self.body.position.x
        ys = other.y - self.body.position.y
        theta = atan2(ys,xs)
        deg = degrees(theta)
        if deg < 0:
            deg += 360
        return deg    
    
    def get_random_target(self,x1=50,y1=50,x2=640,y2=480):
       # width, height = self.window.get_size()
        x = randint(abs(x1),abs(x2))
        y = randint(abs(y1),abs(y2))
        if x > 640: x = 639
        if x < 0: x = 1
        if y > 480: y = 379
        if y < 0 : y =1
        return Vec2d(x,y)
    
    def set_new_target(self):
        self.body.reset_forces()
        radius = 50
        now_pos = self.body.position

        pos = self.get_random_target()
        
        while (pos.get_distance(self.lawn_mower.body.position) < 100):
            pos = self.get_random_target()
            
        self.target_vec2d = pos
        self.target_angle = radians(self.get_angle_to(self.target_vec2d))